@@ -18,6 +18,7 @@
 #include "aht.h"
 #include "antigetopt.h"
 #include "sleep.h"
+#include "blacklist.h"
 
 /* Max length of an error stored in the visitors handle */
 #define VI_ERROR_MAX 1024
@@ -32,7 +33,7 @@
 /* Version as a string */
 #define VI_DATE_MAX 64
 /* Max length of a log entry date */
-#define VI_VERSION_STR "0.5"
+#define VI_VERSION_STR "0.61"
 
 /*------------------------------- data structures ----------------------------*/
 
@@ -42,6 +43,7 @@ struct vih {
 	int endt;
 	int processed;
 	int invalid;
+        int blacklisted;
 	int hour[24];
 	int weekday[7];
 	int weekdayhour[7][24]; /* hour and weekday combined data */
@@ -57,6 +59,7 @@ struct vih {
 	struct hashtable referersage;
 	struct hashtable date;
 	struct hashtable googledate;
+        struct hashtable adsensed;
 	struct hashtable month;
 	struct hashtable googlemonth;
 	struct hashtable agents;
@@ -69,6 +72,7 @@ struct vih {
 	struct hashtable os;
 	struct hashtable browsers;
 	struct hashtable robots;
+        struct hashtable googlehumanlanguage;
 	char *error;
 };
 
@@ -121,6 +125,7 @@ int Config_max_images = 20;
 int Config_max_error404 = 20;
 int Config_max_agents = 20;
 int Config_max_googled = 20;
+int Config_max_adsensed = 20;
 int Config_max_google_keyphrases = 20;
 int Config_max_google_keyphrases_age = 20;
 int Config_max_trails = 20;
@@ -130,6 +135,7 @@ int Config_process_agents = 0;
 int Config_process_google = 0;
 int Config_process_google_keyphrases = 0;
 int Config_process_google_keyphrases_age = 0;
+int Config_process_google_human_language = 0;
 int Config_process_web_trails = 0;
 int Config_process_weekdayhour_map = 0;
 int Config_process_monthday_map = 0;
@@ -150,6 +156,7 @@ int Config_stream_mode = 0;
 int Config_update_every = 60*10; /* update every 10 minutes for default. */
 int Config_reset_every = 0;	/* never reset for default */
 int Config_time_delta = 0;	/* adjustable time difference */
+int Config_filter_spam = 0;
 char *Config_output_file = NULL; /* stdout if not set. */
 struct outputmodule *Output = NULL; /* intialized to 'text' in main() */
 
@@ -176,6 +183,50 @@ int vi_is_google_link(char *s)
 	return !strncmp(s, "http://www.google.", 18);
 }
 
+/* Returns non-zero if the user agent appears to be the GoogleBot. */
+int vi_is_googlebot_agent(char *agent) {
+	if (strstr(agent, "Googlebot") ||
+            strstr(agent, "googlebot")) return 1;
+        return 0;
+}
+
+/* Returns non-zero if the user agent appears to be the Mediapartners-Google. */
+int vi_is_adsensebot_agent(char *agent) {
+	if (strstr(agent, "Mediapartners-Google")) return 1;
+        return 0;
+}
+
+int vi_is_yahoobot_agent(char *agent) {
+        if (strstr(agent, "Yahoo! Slurp")) return 1;
+        return 0;
+}
+
+int vi_is_msbot_agent(char *agent) {
+        if (strstr(agent, "msn.com/msnbot.htm")) return 1;
+        return 0;
+}
+
+/* Try to guess if a given agent string is about a crawler/bot
+ * of some time. This function MUST be conservative, because
+ * false negatives are acceptable while false positives arent. */
+int vi_is_genericbot_agent(char *agent) {
+        if (strstr(agent, "crawler") ||
+            strstr(agent, "Crawler") ||
+            strstr(agent, "bot/") ||
+            strstr(agent, "Bot/") ||
+            strstr(agent, "bot.htm") ||
+            strstr(agent, "+http://")) return 1;
+        return 0;
+}
+
+int vi_is_bot_agent(char *agent) {
+    if (vi_is_googlebot_agent(agent) ||
+        vi_is_adsensebot_agent(agent) ||
+        vi_is_yahoobot_agent(agent) ||
+        vi_is_msbot_agent(agent)) return 1;
+    return 0;
+}
+
 /* Returns non-zero if the url matches some user-specified prefix.
  * being a link "internal" to the site. Otherwise zero is returned.
  *
@@ -536,6 +587,23 @@ int vi_strlcat(char *dst, const char *src, int siz)
         return(dlen + (s - src));       /* count does not include NUL */
 }
 
+/* Returns non-zero if the url matches one of the keywords in
+ * blacklist.h, otherwise zero is returned. Warning!!! This function
+ * run time is proportional to the size of blacklist.h, so it is
+ * very slow. */
+int vi_is_blacklisted_url(struct vih *vih, char *url)
+{
+    unsigned int i;
+
+    for (i = 0; i < VI_BLACKLIST_LEN; i++) {
+        if (strstr(url, vi_blacklist[i])) {
+            vih->blacklisted++;
+            return 1;
+        }
+    }
+    return 0;
+}
+
 /*-------------------------- visitors handler functions --------------------- */
 /* Init the hashtable with methods suitable for an "occurrences counter" */
 void vi_ht_init(struct hashtable *ht)
@@ -578,6 +646,7 @@ void vi_reset_hashtables(struct vih *vih)
 	ht_destroy(&vih->referersage);
 	ht_destroy(&vih->agents);
 	ht_destroy(&vih->googled);
+	ht_destroy(&vih->adsensed);
 	ht_destroy(&vih->googlekeyphrases);
 	ht_destroy(&vih->googlekeyphrasesage);
 	ht_destroy(&vih->googlevisits);
@@ -590,6 +659,7 @@ void vi_reset_hashtables(struct vih *vih)
 	ht_destroy(&vih->month);
 	ht_destroy(&vih->googlemonth);
 	ht_destroy(&vih->robots);
+	ht_destroy(&vih->googlehumanlanguage);
 }
 
 /* Reset handler informations to support --reset option in
@@ -614,6 +684,7 @@ struct vih *vi_new(void)
 	vih->startt = vih->endt = time(NULL);
 	vih->processed = 0;
 	vih->invalid = 0;
+        vih->blacklisted = 0;
 	vi_reset_combined_maps(vih);
 	vih->error = NULL;
 	vi_ht_init(&vih->visitors);
@@ -627,6 +698,7 @@ struct vih *vi_new(void)
 	vi_ht_init(&vih->referersage);
 	vi_ht_init(&vih->agents);
 	vi_ht_init(&vih->googled);
+	vi_ht_init(&vih->adsensed);
 	vi_ht_init(&vih->googlevisits);
 	vi_ht_init(&vih->googlekeyphrases);
 	vi_ht_init(&vih->googlekeyphrasesage);
@@ -639,6 +711,7 @@ struct vih *vi_new(void)
 	vi_ht_init(&vih->googledate);
 	vi_ht_init(&vih->googlemonth);
 	vi_ht_init(&vih->robots);
+	vi_ht_init(&vih->googlehumanlanguage);
 	return vih;
 }
 
@@ -811,6 +884,7 @@ int vi_parse_line(struct logline *ll, char *l)
 {
 	char *date, *hour, *timezone, *host, *agent, *req, *ref, *p;
 	char *agent_start = NULL, *req_end = NULL, *ref_end = NULL;
+        int agent_without_parens = 0;
 
 	/* Seek the start of the different components */
 
@@ -819,10 +893,44 @@ int vi_parse_line(struct logline *ll, char *l)
 	/* date */
 	if ((date = strchr(l, '[')) == NULL) return 1;
 	date++;
-	/* agent */
+	/* Identify user-agent start char. */
 	if ((agent = strchr(l, '(')) == NULL) {
-		agent = "";
+                /* Bad... user agent without (...) string, makes
+                 * the detection a bit slower and guessworkish. */
+
+                /* Check if the count of '"' chars in the string
+                 * is equal to six. If so, it's very likely that the
+                 * last field inside "" is the User Agent string, so
+                 * we get it. */
+                char *aux = l, *last = NULL;
+                int count = 0;
+               
+                /* Count '"' chars, save the last occurence found. */
+                while (*aux) {
+                    if (*aux == '"') {
+                        count++;
+                        last = aux;
+                    }
+                    aux++;
+                }
+
+                if (count == 6) {
+                    /* Ok! it seems like Combined log format.
+                     * Set a flag and get it later when the
+                     * rest of the log file is splitted. Now it's
+                     * too early to add \0 chars inside the line. */
+                    agent_without_parens = 1;
+                    agent_start = last-1;
+                    while(*agent_start != '"')
+                        agent_start--;
+                } else {
+                    /* No way... no user agent detected in this line. */
+		    agent = "";
+                }
 	} else {
+                /* User agent with () inside. Simple to detect, just
+                 * search the left and the right '"' chars enclosing
+                 * it. */
 		p = agent;
 		while (p >= l) {
 			if (*p == '"') {
@@ -891,7 +999,18 @@ int vi_parse_line(struct logline *ll, char *l)
 		*p = '\0';
 	}
 	/* agent */
-	if ((p = strchr(agent, ')')) == NULL) {
+        if (agent_without_parens) {
+            /* User agent without (...) inside in a string with six '"' chars.
+             * Just search for the end. */
+            char *aux = strchr(agent_start+1, '"');
+            if (!aux) {
+                /* No way! */
+                agent = "";
+            } else {
+                *aux = '\0';
+                agent = agent_start+1;
+            }
+        } else if ((p = strchr(agent, ')')) == NULL) {
 		agent = "";
 	} else {
 		char *aux;
@@ -952,12 +1071,18 @@ int vi_process_visitors_per_day(struct vih *vih, char *host, char *agent, char *
 	int res, host_len, agent_len, date_len, hash_len;
         unsigned long h;
 
+        /* Ignore visits from Bots */
+        if (vi_is_bot_agent(agent)) {
+            if (seen != NULL) seen = 0;
+            return 0;
+        }
+
         /* Build an unique identifier for this visit
          * adding together host, date and hash(user agent) */
 	host_len = strlen(host);
 	agent_len = strlen(agent);
 	date_len = strlen(date);
-        h = djb_hash(agent, agent_len);
+        h = djb_hash((unsigned char*) agent, agent_len);
         sprintf(buf, "%lu", h);
         hash_len = strlen(buf);
 	if (host_len+agent_len+date_len+4 > VI_LINE_MAX)
@@ -969,6 +1094,7 @@ int vi_process_visitors_per_day(struct vih *vih, char *host, char *agent, char *
 	*p++ = '|';
 	memcpy(p, buf, hash_len); p += hash_len;
 	*p = '\0';
+        /* fprintf(stderr, "%s\n", visday); */
 
 	if (Config_process_monthly_visitors) {
 		/* Skip the day number. */
@@ -1019,6 +1145,10 @@ int vi_process_referer(struct vih *vih, char *ref, time_t age)
 {
 	int res;
 
+        /* Check the url against the blacklist if needed
+         * this can be very slow... */
+        if (Config_filter_spam && vi_is_blacklisted_url(vih, ref))
+            return 0;
 	/* Don't count internal referer (specified by the user
 	 * using --prefix options), nor google referers. */
 	if (vi_is_internal_link(ref))
@@ -1125,11 +1255,16 @@ int vi_process_browsers(struct vih *vih, char *agent)
 	 * so it must be before Gecko. */
 	char *browserslist[] = {
 		"Opera", NULL,
-		"MSIE", "Explorer",
+		"MSIE 4", "Explorer 4.x",
+		"MSIE 5", "Explorer 5.x",
+		"MSIE 6", "Explorer 6.x",
+		"MSIE", "Explorer unknown version",
 		"Safari", NULL,
 		"Konqueror", NULL,
 		"Galeon", NULL,
-		"Gecko", "Netscape/Mozilla",
+		"Firefox", NULL,
+		"MultiZilla", NULL,
+		"Gecko", "Other Mozilla based",
 		"Wget", NULL,
 		"Lynx", NULL,
 		"Links ", "Links",
@@ -1137,7 +1272,17 @@ int vi_process_browsers(struct vih *vih, char *agent)
 		"Elinks ", "Links",
 		"w3m", "W3M",
 		"NATSU-MICAN", NULL,
-		"www.googlebot.com", "GoogleBot",
+		"msnbot", "MSNbot",
+		"Slurp", "Yahoo Slurp",
+		"Jeeves", "Ask Jeeves",
+		"ZyBorg", NULL,
+		"asteria", NULL,
+		"contype", "Explorer",
+		"Gigabot", NULL,
+		"Windows-Media-Player", "Windows-MP",
+		"NSPlayer", NULL,
+		"Googlebot", "GoogleBot",
+		"googlebot", "GoogleBot",
 		"", "Unknown",
 		NULL, NULL,
 	};
@@ -1148,9 +1293,12 @@ int vi_process_browsers(struct vih *vih, char *agent)
  * Return non-zero on out of memory. */
 int vi_process_googled(struct vih *vih, char *req, char *agent, time_t age)
 {
-	if (!strstr(agent, "Googlebot") &&
-            !strstr(agent, "googlebot")) return 0;
-	return vi_replace_if_newer(&vih->googled, req, age);
+        if (vi_is_googlebot_agent(agent)) {
+	    return vi_replace_if_newer(&vih->googled, req, age);
+        } else if (vi_is_adsensebot_agent(agent)) {
+	    return vi_replace_if_newer(&vih->adsensed, req, age);
+        }
+        return 0;
 }
 
 /* Process accesses with the referer from google.
@@ -1164,6 +1312,19 @@ int vi_process_google_keyphrases(struct vih *vih, char *ref, time_t age)
 	char buf[64];
 
 	if (!vi_is_google_link(ref)) return 0;
+        /* Try to process gogoe human language info first. */
+        if (Config_process_google_human_language) {
+            s = strstr(ref+18, "&hl=");
+            if (s == NULL) s = strstr(ref+18, "?hl=");
+            if (s && s[4] && s[5]) {
+                buf[0] = s[4];
+                buf[1] = s[5];
+                buf[2] = '\0';
+	        if (vi_counter_incr(&vih->googlehumanlanguage, buf) == 0)
+                    return 1;
+            }
+        }
+
 	/* It's possible to start the search for the query 18 chars
 	 * after the start of the referer because all the
 	 * google links will start with "http://www.google.". */
@@ -1243,7 +1404,7 @@ int vi_process_tld(struct vih *vih, char *hostname)
 	int res;
 
 	if (vi_is_numeric_address(hostname)) {
-		tld = "unresolved IP";
+		tld = "numeric IP";
 	} else {
 		tld = strrchr(hostname, '.');
 		if (!tld) return 0;
@@ -1467,7 +1628,7 @@ void om_text_print_bar(FILE *fp, int max, int tot, int this, int cols,
 
 void om_text_print_numkeybar_entry(FILE *fp, char *key, int max, int tot, int this)
 {
-	fprintf(fp, "   %-11s: %-9d |", key, this);
+	fprintf(fp, "   %-12s: %-9d |", key, this);
 	om_text_print_bar(fp, max, tot, this, 44, '#', ' ');
 	fprintf(fp, "\n");
 }
@@ -1863,7 +2024,7 @@ void om_html_print_bidimentional_map(FILE *fp, int xlen, int ylen,
 			r = (0xAA*val)/max;
 			g = (0xBB*val)/max;
 			b = (0xFF*val)/max;
-			fprintf(fp, "<td style=\"background-color: #%02X%02X%02X;\">&nbsp;</td>", r, g, b);
+			fprintf(fp, "<td style=\"background-color: #%02X%02X%02X;\">&nbsp;</td>\n", r, g, b);
 		}
 		fprintf(fp, "</tr>\n");
 	}
@@ -1916,9 +2077,9 @@ void vi_print_statistics(struct vih *vih)
 
 	if (elapsed == 0) elapsed++;
 	fprintf(stderr, "--\n%d lines processed in %ld seconds\n"
-	       "%d invalid lines\n",
+	       "%d invalid lines, %d blacklisted referers\n",
 			vih->processed, (long) elapsed,
-			vih->invalid);
+			vih->invalid, vih->blacklisted);
 }
 
 void vi_print_hours_report(FILE *fp, struct vih *vih)
@@ -2071,6 +2232,7 @@ void vi_print_visits_report(FILE *fp, struct vih *vih)
 		Output->print_numkeybar_entry(fp, key, max, tot, value);
 	}
 	free(table);
+        Output->print_hline(fp);
 
 	/* Montly */
 	if (Config_process_monthly_visitors == 0) return;
@@ -2131,6 +2293,7 @@ void vi_print_googlevisits_report(FILE *fp, struct vih *vih)
 		Output->print_numkeycomparativebar_entry(fp, key, value, googlevalue);
 	}
 	free(table);
+        Output->print_hline(fp);
 
 	/* Montly */
 	if (Config_process_monthly_visitors == 0) return;
@@ -2273,7 +2436,7 @@ void vi_print_referers_report(FILE *fp, struct vih *vih)
 	vi_print_generic_keyval_report(
 			fp,
 			"Referers",
-			"Referers ordered by visits (referers from google excluded)",
+			"Referers ordered by visits (google excluded)",
 			"Different referers",
 			Config_max_referers,
 			&vih->referers,
@@ -2310,7 +2473,7 @@ void vi_print_pageviews_report(FILE *fp, struct vih *vih)
 			fp,
 			"Pageviews per visit",
 			"Number of pages requested per visit",
-			"Only extensions like .html, .htm, .php, .asp, ..., are counted. Reported ranges:",
+			"Only documents are counted (not images). Reported ranges:",
 			100,
 			&vih->pageviews_grouped,
 			qsort_cmp_long_value);
@@ -2460,6 +2623,18 @@ void vi_print_googled_report(FILE *fp, struct vih *vih)
 			qsort_cmp_time_value);
 }
 
+void vi_print_adsensed_report(FILE *fp, struct vih *vih)
+{
+	vi_print_generic_keytime_report(
+			fp,
+			"Adsensed pages",
+			"Pages accessed by the Adsense crawler, last access reported",
+			"Number of pages adsensed",
+			Config_max_adsensed,
+			&vih->adsensed,
+			qsort_cmp_time_value);
+}
+
 void vi_print_referers_age_report(FILE *fp, struct vih *vih)
 {
 	vi_print_generic_keytime_report(
@@ -2484,6 +2659,18 @@ void vi_print_google_keyphrases_age_report(FILE *fp, struct vih *vih)
 			qsort_cmp_time_value);
 }
 
+void vi_print_google_human_language_report(FILE *fp, struct vih *vih)
+{
+	vi_print_generic_keyval_report(
+			fp,
+			"Google Human Language",
+			"The 'hl' field in the query string of google searches",
+			"Different human languages",
+			1000,
+			&vih->googlehumanlanguage,
+			qsort_cmp_long_value);
+}
+
 void vi_print_information_report(FILE *fp, struct vih *vih)
 {
 	char buf[VI_LINE_MAX];
@@ -2518,8 +2705,10 @@ void vi_print_report_links(FILE *fp)
 	"404 Errors", &Config_process_error404,
 	"Domains", &Config_process_tld,
 	"Googled pages", &Config_process_google,
+	"Adsensed pages", &Config_process_google,
 	"Google Keyphrases", &Config_process_google_keyphrases,
 	"Google Keyphrases by first time", &Config_process_google_keyphrases_age,
+	"Google Human Language", &Config_process_google_human_language,
 	"Web trails", &Config_process_web_trails,
 	"Weekday distribution", NULL,
 	"Hours distribution", NULL,
@@ -2709,6 +2898,8 @@ int vi_print_report(char *of, struct vih *vih)
 	if (Config_process_google) {
 		vi_print_googled_report(fp, vih);
 		vi_print_hline(fp);
+		vi_print_adsensed_report(fp, vih);
+		vi_print_hline(fp);
 	}
 	if (Config_process_google_keyphrases) {
 		vi_print_google_keyphrases_report(fp, vih);
@@ -2718,6 +2909,10 @@ int vi_print_report(char *of, struct vih *vih)
 		vi_print_google_keyphrases_age_report(fp, vih);
 		vi_print_hline(fp);
 	}
+        if (Config_process_google_human_language) {
+		vi_print_google_human_language_report(fp, vih);
+		vi_print_hline(fp);
+        }
 	if (Config_process_web_trails) {
 		vi_print_trails_report(fp, vih);
 		vi_print_hline(fp);
@@ -2818,7 +3013,7 @@ void vi_stream_mode(struct vih *vih)
 /* ----------------------------------- main --------------------------------- */
 
 /* command line switche IDs */
-enum { OPT_MAXREFERERS, OPT_MAXPAGES, OPT_MAXIMAGES, OPT_USERAGENTS, OPT_ALL, OPT_MAXLINES, OPT_GOOGLE, OPT_MAXGOOGLED, OPT_MAXUSERAGENTS, OPT_OUTPUT, OPT_VERSION, OPT_HELP, OPT_PREFIX, OPT_TRAILS, OPT_GOOGLEKEYPHRASES, OPT_GOOGLEKEYPHRASESAGE, OPT_MAXGOOGLEKEYPHRASES, OPT_MAXGOOGLEKEYPHRASESAGE, OPT_MAXTRAILS, OPT_GRAPHVIZ, OPT_WEEKDAYHOUR_MAP, OPT_MONTHDAY_MAP, OPT_REFERERSAGE, OPT_MAXREFERERSAGE, OPT_TAIL, OPT_TLD, OPT_MAXTLD, OPT_STREAM, OPT_OUTPUTFILE, OPT_UPDATEEVERY, OPT_RESETEVERY, OPT_OS, OPT_BROWSERS, OPT_ERROR404, OPT_MAXERROR404, OPT_TIMEDELTA, OPT_PAGEVIEWS, OPT_ROBOTS, OPT_MAXROBOTS, OPT_GRAPHVIZ_ignorenode_GOOGLE, OPT_GRAPHVIZ_ignorenode_EXTERNAL, OPT_GRAPHVIZ_ignorenode_NOREFERER};
+enum { OPT_MAXREFERERS, OPT_MAXPAGES, OPT_MAXIMAGES, OPT_USERAGENTS, OPT_ALL, OPT_MAXLINES, OPT_GOOGLE, OPT_MAXGOOGLED, OPT_MAXUSERAGENTS, OPT_OUTPUT, OPT_VERSION, OPT_HELP, OPT_PREFIX, OPT_TRAILS, OPT_GOOGLEKEYPHRASES, OPT_GOOGLEKEYPHRASESAGE, OPT_MAXGOOGLEKEYPHRASES, OPT_MAXGOOGLEKEYPHRASESAGE, OPT_MAXTRAILS, OPT_GRAPHVIZ, OPT_WEEKDAYHOUR_MAP, OPT_MONTHDAY_MAP, OPT_REFERERSAGE, OPT_MAXREFERERSAGE, OPT_TAIL, OPT_TLD, OPT_MAXTLD, OPT_STREAM, OPT_OUTPUTFILE, OPT_UPDATEEVERY, OPT_RESETEVERY, OPT_OS, OPT_BROWSERS, OPT_ERROR404, OPT_MAXERROR404, OPT_TIMEDELTA, OPT_PAGEVIEWS, OPT_ROBOTS, OPT_MAXROBOTS, OPT_GRAPHVIZ_ignorenode_GOOGLE, OPT_GRAPHVIZ_ignorenode_EXTERNAL, OPT_GRAPHVIZ_ignorenode_NOREFERER, OPT_GOOGLEHUMANLANGUAGE, OPT_FILTERSPAM, OPT_MAXADSENSED};
 
 /* command line switches definition:
  * the rule with short options is to take upper case the
@@ -2829,6 +3024,7 @@ static struct ago_optlist visitors_optlist[] = {
 	{ 'G',	"google",		OPT_GOOGLE,		AGO_NOARG},
 	{ 'K',	"google-keyphrases",	OPT_GOOGLEKEYPHRASES,	AGO_NOARG},
 	{ 'Z',	"google-keyphrases-age", OPT_GOOGLEKEYPHRASESAGE, AGO_NOARG},
+        { 'H',  "google-human-language", OPT_GOOGLEHUMANLANGUAGE, AGO_NOARG},
 	{ 'U',	"user-agents",		OPT_USERAGENTS,		AGO_NOARG},
 	{ 'W',  "weekday-hour-map",	OPT_WEEKDAYHOUR_MAP,	AGO_NOARG},
 	{ 'M',  "month-day-map",	OPT_MONTHDAY_MAP,	AGO_NOARG},
@@ -2851,6 +3047,7 @@ static struct ago_optlist visitors_optlist[] = {
 	{ 'u',	"max-useragents",	OPT_MAXUSERAGENTS,	AGO_NEEDARG},
 	{ 't',	"max-trails",		OPT_MAXTRAILS,		AGO_NEEDARG},
 	{ 'g',	"max-googled",		OPT_MAXGOOGLED,		AGO_NEEDARG},
+	{ '\0',	"max-adsensed",		OPT_MAXADSENSED,	AGO_NEEDARG},
 	{ 'k',	"max-google-keyphrases",OPT_MAXGOOGLEKEYPHRASES,AGO_NEEDARG},
 	{ 'z',	"max-google-keyphrases-age",OPT_MAXGOOGLEKEYPHRASESAGE,
 		AGO_NEEDARG},
@@ -2869,6 +3066,7 @@ static struct ago_optlist visitors_optlist[] = {
 	{ 'v',  "version",		OPT_VERSION,		AGO_NOARG},
 	{ '\0', "tail",			OPT_TAIL,		AGO_NOARG},
 	{ '\0', "time-delta",		OPT_TIMEDELTA,		AGO_NEEDARG},
+        { '\0', "filter-spam",          OPT_FILTERSPAM,         AGO_NOARG},
 	{ 'h',	"help",			OPT_HELP,		AGO_NOARG},
 	AGO_LIST_TERM
 };
@@ -2890,6 +3088,7 @@ void visitors_show_help(void)
 				(visitors_optlist[i].ao_flags & AGO_NEEDARG) ?
 					"<argument>" : "");
 	}
+        printf("\nNOTE: --filter-spam can be *very* slow. Use with care.\n\n");
 	printf("For more information visit http://www.hping.org/visitors\n"
 	       "Visitors is Copyright(C) 2004,2005 Salvatore Sanfilippo <antirez@invece.org>\n");
 }
@@ -2939,6 +3138,9 @@ int main(int argc, char **argv)
 		case OPT_MAXGOOGLED:
 			Config_max_googled = atoi(ago_optarg);
 			break;
+		case OPT_MAXADSENSED:
+			Config_max_adsensed = atoi(ago_optarg);
+			break;
 		case OPT_MAXGOOGLEKEYPHRASES:
 			Config_max_google_keyphrases = atoi(ago_optarg);
 			break;
@@ -2966,6 +3168,10 @@ int main(int argc, char **argv)
 		case OPT_GOOGLEKEYPHRASESAGE:
 			Config_process_google_keyphrases_age = 1;
 			break;
+		case OPT_GOOGLEHUMANLANGUAGE:
+                        Config_process_google_keyphrases = 1;
+			Config_process_google_human_language = 1;
+			break;
 		case OPT_TLD:
 			Config_process_tld = 1;
 			break;
@@ -2989,6 +3195,7 @@ int main(int argc, char **argv)
 			Config_process_google = 1;
 			Config_process_google_keyphrases = 1;
 			Config_process_google_keyphrases_age = 1;
+			Config_process_google_human_language = 1;
 			Config_process_weekdayhour_map = 1;
 			Config_process_monthday_map = 1;
 			Config_process_referers_age = 1;
@@ -3021,6 +3228,7 @@ int main(int argc, char **argv)
 				Config_max_error404 = aux;
 				Config_max_agents = aux;
 				Config_max_googled = aux;
+				Config_max_adsensed = aux;
 				Config_max_trails = aux;
 				Config_max_google_keyphrases = aux;
 				Config_max_google_keyphrases_age = aux;
@@ -3080,6 +3288,9 @@ int main(int argc, char **argv)
 		case OPT_TIMEDELTA:
 			Config_time_delta = atoi(ago_optarg);
 			break;
+                case OPT_FILTERSPAM:
+                        Config_filter_spam = 1;
+                        break;
 		case AGO_ALONE:
 			if (filenamec < VI_FILENAMES_MAX)
 				filenames[filenamec++] = ago_optarg;
